import React, { useMemo, useEffect, useState, useContext } from "react";
import { useNavigate } from "react-router-dom";
import { useTypingTimer } from "../../common/SearchBoxFilter/TypingTimer";
import { useCustomSnackbar } from "../../common/snackbar";
import { RepositoryOptions, WITHOUT_REPO } from "./RepositoryOptions";
import { RepositoryName } from "./RepositoryName";
import { RootPaper } from "@ess-ics/ce-ui-common/dist/components/common/container/RootPaper";
import {
  Alert,
  Autocomplete,
  Button,
  CircularProgress,
  LinearProgress,
  Stack,
  TextField,
  Typography
} from "@mui/material";
import { useAPIMethod } from "@ess-ics/ce-ui-common";
import { apiContext } from "../../../api/DeployApi";
import { getErrorMessage } from "../../common/Helper";

const createRequestParams = (query) => {
  return {
    query: query
  };
};

export function CreateIOC() {
  const navigate = useNavigate();
  const showSnackBar = useCustomSnackbar();
  const [namingEntity, setNamingEntity] = useState({});
  const [gitProject, setGitProject] = useState({});
  const [nameQuery, onNameKeyUp] = useTypingTimer({ interval: 500 });
  const [repoQuery, onRepoKeyUp] = useTypingTimer({ interval: 500 });
  const [selectedRepoOption, setSelectedRepoOption] = useState(WITHOUT_REPO);
  const [repoName, setRepoName] = useState("");

  const client = useContext(apiContext);

  const requestParams = useMemo(
    () => createRequestParams(repoQuery),
    [repoQuery]
  );

  const {
    value: allowedGitProjects,
    wrapper: getAllowedGitProjects,
    loading: loadingAllowedGitProjects
  } = useAPIMethod({
    fcn: client.apis.Git.listProjects,
    params: requestParams,
    call: false
  });

  const {
    value: ioc,
    wrapper: createIoc,
    loading,
    error
  } = useAPIMethod({
    fcn: client.apis.IOCs.createIoc,
    call: false
  });

  const {
    value: names,
    wrapper: getNames,
    loading: loadingNames
  } = useAPIMethod({
    fcn: client.apis.Names.fetchIOCByName,
    call: false
  });

  // create the ioc on form submit
  const onSubmit = (event) => {
    event.preventDefault();
    createIoc(
      {},
      {
        requestBody: {
          gitProjectId: gitProject.id,
          namingUuid: namingEntity ? namingEntity.uuid : undefined,
          repository_name: repoName ? repoName : undefined
        }
      }
    );
  };

  const handleSelectRepoOption = (option) => {
    setSelectedRepoOption(option);
    setNamingEntity({});
    setGitProject({});
  };

  // fetch new names when name query changes
  useEffect(() => {
    if (nameQuery) {
      getNames(nameQuery);
    }
  }, [nameQuery, getNames]);

  useEffect(() => {
    if (repoQuery) {
      getAllowedGitProjects();
    }
  }, [repoQuery, getAllowedGitProjects]);

  // navigate home once ioc created
  useEffect(() => {
    if (ioc) {
      showSnackBar(
        selectedRepoOption === WITHOUT_REPO
          ? "IOC created"
          : "IOC created with a repository",
        "success"
      );
      navigate(`/iocs/${ioc.id}?&tab=Management`);
    }
  }, [ioc, showSnackBar, selectedRepoOption, navigate]);

  return (
    <RootPaper
      sx={{ display: "flex", flexDirection: "column", alignItems: "center" }}
    >
      <Stack
        component="form"
        onSubmit={onSubmit}
        gap={2}
        aria-live="polite"
        width="600px"
      >
        <Typography variant="h2">Create new IOC</Typography>
        <RepositoryOptions
          selectedRepoOption={selectedRepoOption}
          onSelectRepoOption={handleSelectRepoOption}
        />
        <Autocomplete
          autoHighlight
          id="nameAutocomplete"
          value={namingEntity}
          options={nameQuery ? names ?? [] : []}
          loading={loadingNames}
          clearOnBlur={false}
          getOptionLabel={(option) => option?.name ?? ""}
          renderInput={(params) => (
            <TextField
              {...params}
              label="IOC name"
              variant="outlined"
              required
              InputProps={{
                ...params.InputProps,
                endAdornment: (
                  <React.Fragment>
                    {loadingNames ? (
                      <CircularProgress
                        color="inherit"
                        size={20}
                      />
                    ) : null}
                    {params.InputProps.endAdornment}
                  </React.Fragment>
                )
              }}
              disabled={loading}
              helperText={namingEntity ? namingEntity.description : ""}
            />
          )}
          onChange={(_, value) => setNamingEntity(value)}
          onInputChange={(event) => event && onNameKeyUp(event.nativeEvent)}
          autoSelect
        />
        {selectedRepoOption === WITHOUT_REPO ? (
          <Autocomplete
            autoHighlight
            id="gitId"
            value={gitProject}
            options={repoQuery || gitProject ? allowedGitProjects ?? [] : []}
            loading={loadingAllowedGitProjects}
            clearOnBlur={false}
            getOptionLabel={(option) => {
              return option?.url ?? "";
            }}
            onChange={(_, value) => setGitProject(value)}
            onInputChange={(event) => event && onRepoKeyUp(event.nativeEvent)}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Git repository"
                variant="outlined"
                fullWidth
                required
                InputProps={{
                  ...params.InputProps,
                  endAdornment: (
                    <React.Fragment>
                      {loadingAllowedGitProjects ? (
                        <CircularProgress
                          color="inherit"
                          size={20}
                        />
                      ) : null}
                      {params.InputProps.endAdornment}
                    </React.Fragment>
                  )
                }}
                disabled={loading}
              />
            )}
            autoSelect
          />
        ) : (
          <RepositoryName
            repoName={repoName}
            onRepoNameChange={(name) => setRepoName(name)}
          />
        )}

        {error ? (
          <Alert severity="error">{getErrorMessage(error)}</Alert>
        ) : null}
        {loading ? (
          <LinearProgress
            aria-busy="true"
            aria-label="Creating IOC, please wait"
          />
        ) : null}
        <Stack
          direction="row"
          justifyContent="flex-end"
        >
          <Button
            onClick={() => navigate("/")}
            color="primary"
            disabled={loading}
          >
            Cancel
          </Button>
          <Button
            color="primary"
            variant="contained"
            type="submit"
            disabled={
              loading || !namingEntity || selectedRepoOption === WITHOUT_REPO
                ? !gitProject
                : !repoName
            }
          >
            Create
          </Button>
        </Stack>
      </Stack>
    </RootPaper>
  );
}
