import React, { useContext, useEffect, useMemo, useCallback } from "react";
import { string, number, boolean, func } from "prop-types";
import { serialize } from "../../../components/common/URLState/URLState";
import { getErrorMessage } from "../../../components/common/Helper";
import { apiContext } from "../../../api/DeployApi";
import {
  SimpleAccordion,
  useAPIMethod,
  usePagination
} from "@ess-ics/ce-ui-common";
import { JobTable } from "../../../components/Job";
import { Alert, Typography } from "@mui/material";

const propTypes = {
  hostId: string.isRequired,
  rows: number,
  page: number,
  expanded: boolean,
  onUrlStateChange: func
};

const rowsPerPage = [20, 50];

export const HostJobsSection = ({
  hostId,
  rows,
  page,
  expanded,
  onUrlStateChange
}) => {
  const client = useContext(apiContext);

  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: rowsPerPage,
    initLimit: rows,
    initPage: page
  });

  const {
    value: hostLog,
    wrapper: getHostLog,
    dataReady,
    error,
    loading,
    abort: abortGetHostLog
  } = useAPIMethod({
    fcn: client.apis.Deployments.listOperations,
    call: false,
    params: useMemo(
      () => ({ host_id: hostId, ...pagination }),
      [hostId, pagination]
    )
  });

  const onPage = useCallback(
    (params) => {
      setPagination(params);
      abortGetHostLog();
      onUrlStateChange({
        job_log_rows: serialize(params.limit),
        job_log_page: serialize(params.page)
      });
    },
    [setPagination, abortGetHostLog, onUrlStateChange]
  );

  useEffect(() => {
    setPagination({ totalCount: hostLog?.totalCount ?? 0 });
  }, [setPagination, hostLog]);

  useEffect(() => {
    if (expanded) {
      getHostLog();
    }
  }, [expanded, pagination, getHostLog]);

  useEffect(() => {
    return () => {
      abortGetHostLog();
    };
  }, [abortGetHostLog]);

  return (
    <SimpleAccordion
      summary={
        <Typography
          variant="h3"
          component="h2"
        >
          Job log
        </Typography>
      }
      onChange={(_, isExpanded) =>
        onUrlStateChange({ job_log_open: serialize(isExpanded) })
      }
    >
      {error ? <Alert severity="error">{getErrorMessage(error)}</Alert> : null}
      {hostLog ? (
        <JobTable
          jobs={!error && hostLog ? hostLog?.operations : null}
          customColumns={[
            { field: "status" },
            { field: "job" },
            { field: "user" }
          ]}
          loading={loading || !dataReady}
          pagination={pagination}
          onPage={onPage}
        />
      ) : null}
    </SimpleAccordion>
  );
};

HostJobsSection.propTypes = propTypes;
