import React, {
  useState,
  useMemo,
  useCallback,
  useContext,
  useEffect
} from "react";
import { Container, Grid, Tabs, Tab } from "@mui/material";
import {
  GlobalAppBarContext,
  RootPaper,
  useAPIMethod,
  usePagination
} from "@ess-ics/ce-ui-common";
import {
  applicationTitle,
  initRequestParams
} from "../../components/common/Helper";
import { SearchBar } from "../../components/common/SearchBar/SearchBar";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";
import { RecordTable } from "../../components/records/RecordTable";
import { apiContext } from "../../api/DeployApi";

export function RecordListView() {
  const { setTitle } = useContext(GlobalAppBarContext);
  useEffect(() => setTitle(applicationTitle("Records")), [setTitle]);

  const client = useContext(apiContext);

  const {
    value: records,
    wrapper: getRecords,
    loading,
    dataReady,
    abort
  } = useAPIMethod({
    fcn: client.apis.Records.findAllRecords,
    call: false
  });

  const [urlState, setUrlState] = useUrlState(
    {
      tab: "0",
      rows: "20",
      page: "0",
      query: ""
    },
    { navigateMode: "replace" }
  );
  const [recordFilter, setRecordFilter] = useState(null);
  // used to request record list again when tab is switched, but request it only once! (totalRecord is a random number that is generated by ChannelFinder)

  const handleTabChange = useCallback(
    (event, tab) => {
      setUrlState((s) =>
        serialize(s.tab) === serialize(tab)
          ? { tab: serialize(tab) }
          : { tab: serialize(tab), page: "0" }
      );

      changeTab(tab);
    },
    [setUrlState]
  );

  const changeTab = (tab) => {
    if (tab === 0) {
      setRecordFilter(null);
    } else if (tab === 1) {
      setRecordFilter("ACTIVE");
    } else if (tab === 2) {
      setRecordFilter("INACTIVE");
    }
  };

  useEffect(() => {
    urlState.tab && changeTab(deserialize(urlState.tab));
  }, [urlState]);

  const urlPagination = useMemo(() => {
    return {
      rows: deserialize(urlState.rows),
      page: deserialize(urlState.page)
    };
  }, [urlState.rows, urlState.page]);

  const setUrlPagination = useCallback(
    ({ rows, page }) => {
      setUrlState({
        rows: serialize(rows),
        page: serialize(page)
      });
    },
    [setUrlState]
  );

  const rowsPerPage = [20, 50];

  const { pagination, setPagination, setTotalCount } = usePagination({
    rowsPerPageOptions: rowsPerPage,
    initLimit: urlPagination.rows ?? rowsPerPage[0],
    initPage: urlPagination.page ?? 0
  });

  // update pagination whenever search result total pages change
  useEffect(() => {
    setTotalCount(records?.totalCount ?? 0);
  }, [records?.totalCount, setTotalCount]);

  // whenever url state changes, update pagination
  useEffect(() => {
    setPagination({ ...urlPagination });
  }, [setPagination, urlPagination]);

  // whenever table pagination internally changes (user clicks next page etc)
  // update the row params
  useEffect(() => {
    setUrlPagination(pagination);
  }, [pagination, setUrlPagination]);

  // Request new search results whenever search or pagination changes
  useEffect(() => {
    let requestParams = initRequestParams(pagination);
    requestParams.pv_status = recordFilter;
    requestParams.text = deserialize(urlState.query);
    getRecords(requestParams);

    return () => {
      abort();
    };
  }, [getRecords, recordFilter, urlState.query, pagination, abort]);

  // Callback for searchbar, called whenever user updates search
  const setSearch = useCallback(
    (query) => {
      setUrlState({ query: serialize(query) });
    },
    [setUrlState]
  );

  // Invoked by Table on change to pagination
  const onPage = (params) => {
    setPagination(params);
    abort();
  };

  let content = (
    <SearchBar
      search={setSearch}
      query={deserialize(urlState.query)}
      loading={loading || !dataReady}
      placeholder="Search"
    >
      <RecordTable
        records={records}
        loading={loading || !dataReady}
        pagination={pagination}
        onPage={onPage}
      />
    </SearchBar>
  );

  return (
    <RootPaper>
      <Grid
        container
        spacing={1}
      >
        <Grid
          container
          spacing={1}
          component={Container}
          justifyContent="space-between"
          alignItems="center"
          style={{ display: "flex" }}
        >
          <Grid item>
            <Tabs
              value={deserialize(urlState.tab)}
              onChange={handleTabChange}
            >
              <Tab label="All" />
              <Tab label="Only active" />
              <Tab label="Only inactive" />
            </Tabs>
          </Grid>
        </Grid>
        <Grid
          item
          xs={12}
          md={12}
        >
          {content}
        </Grid>
      </Grid>
    </RootPaper>
  );
}
